/*******************************************************************
*
* Wifiradio
* Andreas Brietzke, Nagareyama, J
*
* Chip type           : ATmega88
* Clock frequency     : 3,6864 MHz
*
*     usart.c
*
*
*
********************************************************************/

#include "usart.h"
#include <avr/interrupt.h>
#include <string.h>
#include <inttypes.h>
#include <stdlib.h>

// Status and Error Flags
volatile int8_t iUartFlags;
volatile char sInBuffer[BUFFER_LENGHT];
volatile uint8_t iBufferPtr;

// ------------------------------------------------------------ USART

void InitUsart(void) {
   UBRR0H = (uint8_t)(UART_BAUD_CALC(UART_BAUD_RATE,F_CPU)>>8); // set baud rate
   UBRR0L = (uint8_t)UART_BAUD_CALC(UART_BAUD_RATE,F_CPU);
   UCSR0B = (1 << RXEN0) | (1 << TXEN0) | (1 << RXCIE0);        // enable receiver and transmitter and receive INT
//   UCSR0C = (1 << UCSZ01) | (1 << UCSZ00);                      // asynchronous 8E1, no Parity
   do UDR0; while (UCSR0A & (1 << RXC0));                       // flush receiver               
}

ISR(USART_RX_vect) {
char cNextChar;
uint8_t status;

   status = UCSR0A;
   cNextChar = UDR0;

      if (iBufferPtr >= BUFFER_LENGHT) ResetCommand();

      if (!(status & (1<<FE0 | 1<<DOR0 | 1<<UPE0))) {           // no frame error, data overrun, parity error
         if ((cNextChar == 0x0D)) {
            UCSR0B &= ~(1 << RXEN0);                            // End of command. Disable receive INT
            sInBuffer[iBufferPtr++] = 0x00;                     // EOS
            iUartFlags |= FLAG_COMM_COMP;        
         } else {
            sInBuffer[iBufferPtr++] = cNextChar;
         }
      }
}

void ResetCommand(void) {
   iUartFlags &= ~(FLAG_COMM_REC | FLAG_PARM1_REC | FLAG_COMM_COMP);
   iBufferPtr = 0;
   sInBuffer[iBufferPtr] = 0x00;
   UCSR0B |= (1 << RXEN0);                                      // enable receive INT
}

char GetCommand (char *sPar, char *sComm) {
int8_t iRecStrPtr;
char sRecString[BUFFER_LENGHT];
int8_t iTmpBufferPtr;

   iTmpBufferPtr = 0;
   
   if (iUartFlags & FLAG_COMM_COMP ) {                          // parsing until EOS

      iRecStrPtr = 0;
      while ((sInBuffer[iTmpBufferPtr] != 0x00) && (sInBuffer[iTmpBufferPtr] != 0x20)) {    // Command, max. string lenght from ISR
         sComm[iRecStrPtr++] = sInBuffer[iTmpBufferPtr++];
      }
      sComm[iRecStrPtr++] = 0x00;

      iRecStrPtr = 0;
      while ((sInBuffer[iTmpBufferPtr++] != 0x00)) {    // Parm 1
         sPar[iRecStrPtr++] = sInBuffer[iTmpBufferPtr];
      }
      sRecString[iRecStrPtr++] = 0x00;

      return 1;
   } else {
      return 0;
   }
}

// ISR(USART_UDRE_vect) {UsartPuts ("USART_UDRE_vect");}
// ISR(USART_TXC_vect) {UsartPuts ("USART_TXC_vect");}

void UsartPutc(char c) {
int temp_Statusreg;
   temp_Statusreg = SREG;
   while(!(UCSR0A & (1 << UDRE0)));                             // wait until UDR ready
   UDR0 = c;                                                    // send character
   SREG = temp_Statusreg;
}

void UsartPuts (char *s) {                                      //  loop until *s != NULL
   while (*s) {
      UsartPutc(*s);
      s++;
   }
}

char UsartGetc(void) {
   while (!(UCSR0A & (1<<RXC0)));                              // wait for receive
   return UDR0;
}

